@@ -213,11 +213,6 @@ class SupplyCurveAggregationConfig(AnalysisConfig):
         """Get the friction dataset name in friction_fpath."""
         return self.get('friction_dset', None)
 
-    @property
-    def check_excl_layers(self):
-        """Get the check_excl_layers flag."""
-        return self.get('check_excl_layers', False)
-
     @property
     def cap_cost_scale(self):
         """Optional LCOE scaling equation to implement "economies of scale".
@@ -230,14 +225,12 @@ class SupplyCurveAggregationConfig(AnalysisConfig):
         return self.get('cap_cost_scale', None)
 
     @property
-    def max_workers(self):
-        """Get the number of workers to use during computation"""
-        return self.get('max_workers', None)
-
-    @property
-    def points_per_worker(self):
-        """Get the number of sc points to summarize on each worker"""
-        return self.get('points_per_worker', None)
+    def pre_extract_inclusions(self):
+        """Optional flag to pre-extract/compute the inclusion mask from the
+        provided excl_dict, by default False. Typically faster to compute
+        the inclusion mask on the fly with parallel workers.
+        """
+        return self.get('pre_extract_inclusions', False)
 
 
 class SupplyCurveConfig(AnalysisConfig):
@@ -355,8 +348,3 @@ class SupplyCurveConfig(AnalysisConfig):
         Get flag to execute competitive wind directions for offshore sites
         """
         return self.get('offshore_compete', False)
-
-    @property
-    def max_workers(self):
-        """Get the number of workers to use during computation"""
-        return self.get('max_workers', None)
