@@ -86,7 +86,8 @@ def from_config(ctx, config_file, verbose):
         os.makedirs(config.dirout)
 
     # initialize loggers.
-    init_mult(name, config.logdir, modules=['reV', 'rex'], verbose=verbose)
+    init_mult(name, config.logdir, modules=[__name__, 'reV', 'rex'],
+              verbose=verbose)
     cf_files = config.parse_cf_files()
     # Initial log statements
     logger.info('Running reV Econ from config file: "{}"'
@@ -293,7 +294,7 @@ def local(ctx, max_workers, timeout, points_range, verbose):
         dirout = os.path.dirname(cf_file)
 
     # initialize loggers for multiple modules
-    init_mult(name, logdir, modules=['reV', 'rex'],
+    init_mult(name, logdir, modules=[__name__, 'reV', 'rex'],
               verbose=verbose, node=True)
 
     for key, val in ctx.obj.items():
@@ -306,9 +307,9 @@ def local(ctx, max_workers, timeout, points_range, verbose):
     t0 = time.time()
 
     # Execute the Generation module with smart data flushing.
-    Econ.reV_run(points=points,
-                 sam_files=sam_files,
-                 cf_file=cf_file,
+    Econ.reV_run(points,
+                 sam_files,
+                 cf_file,
                  year=year,
                  site_data=site_data,
                  output_request=output_request,
