@@ -81,7 +81,8 @@ def from_config(ctx, config_file, verbose):
         verbose = True
 
     # initialize loggers
-    init_mult(name, config.logdir, modules=['reV', 'rex'], verbose=verbose)
+    init_mult(name, config.logdir, modules=[__name__, 'reV', 'rex'],
+              verbose=verbose)
 
     # Initial log statements
     logger.info('Running reV offshore aggregation from config '
@@ -162,7 +163,7 @@ def direct(ctx, gen_fpath, offshore_fpath, points, sam_files,
 
     if ctx.invoked_subcommand is None:
         t0 = time.time()
-        init_mult(name, log_dir, modules=['reV', 'rex'],
+        init_mult(name, log_dir, modules=[__name__, 'reV', 'rex'],
                   verbose=verbose, node=True)
 
         fpath_out = gen_fpath.replace('.h5', '_offshore.h5')
